<?php
/**
 * WeCodeArt Framework.
 *
 * WARNING: This file is part of the core WeCodeArt Framework. DO NOT edit this file under any circumstances.
 * Please do all modifications in the form of a child theme.
 *
 * @package 	WeCodeArt Framework
 * @subpackage  Utilities\Markup
 * @copyright   Copyright (c) 2019, WeCodeArt Framework
 * @since		3.5
 * @version		3.7.7
 */

namespace WeCodeArt\Utilities;

if ( ! defined( 'ABSPATH' ) ) exit();

/**
 * Markup Utilities Parent Class
 */
class Markup {

	use \WeCodeArt\Singleton; 

	/**
	 * Merge array of attributes with defaults, and apply contextual filter on array.
	 * @since 	3.5
	 *
	 * @param 	string 	$context    The context, to build filter name.
	 * @param 	array  	$attributes Optional. Extra attributes to merge with defaults.
	 * @param 	array  	$args       Optional. Custom data to pass to filter.
	 *
	 * @return 	array 	Merged and filtered attributes.
	 */
	public static function parse_attr( $context, $attributes = [], $args = [] ) {
		$defaults = [
			'class' => sanitize_html_class( $context )
		];

		$attributes = wp_parse_args( $attributes, $defaults );

		// Contextual filter.
		return apply_filters( "wecodeart/filter/attributes/{$context}", $attributes, $context, $args );
	}

	/**
	 * Build list of attributes into a string and apply contextual filter on string.
	 *
	 * @since 	3.5
	 *
	 * @param 	string 	$context    The context, to build filter name.
	 * @param 	array  	$attributes Optional. Extra attributes to merge with defaults.
	 * @param 	array  	$args       Optional. Custom data to pass to filter.
	 *
	 * @return 	string
	 */
	public static function generate_attr( $context, $attributes = [], $args = [] ) {
		$attributes = self::parse_attr( $context, $attributes, $args );
		$output = '';

		if( empty( $attributes ) ) return $output;

		// Cycle through attributes, build tag attribute string.
		foreach( $attributes as $key => $value ) {
			if ( ! $value ) continue;
			if ( true === $value ) $output .= esc_html( $key ) . ' ';
			else $output .= sprintf( '%s="%s" ', esc_html( $key ), esc_attr( $value ) );
		}

		$output = apply_filters( "wecodeart/filter/attributes/{$context}/output", $output, $attributes, $context, $args );
		return trim( $output );
	}

	/**
	 * Return Sortable callback functions based on input.
	 *
	 * @version	3.7.7
	 *
	 * @param	array  	modules	required
	 * @param 	array 	options	required
	 * @param 	boolean	echo	optional
	 *
	 * @return 	string|object 	HTML/WP_Error
	 */
	public static function sortable( array $modules, array $options, $echo = true ) { 
		$_callbacks = array_map( function( $module ) {
			return ( isset( $module['callback'] ) && is_callable( $module['callback'] ) ); 
		}, $modules );

		// Only run if we have callbacks set for all of the modules.
		if( count( array_unique( $_callbacks ) ) !== 1  ) {
			return new \WP_Error( 'wecodeart_markup_has_no_callbacks', 
				__( 'Please define a callback function for each of the modules.', 'wecodeart' ) 
			);
		}

		$functions = [];
		
		// If key from options exists in modules array, loads it's callback functions.
		foreach( $options as $opt ) {
			if( array_key_exists( $opt, $modules ) ) {
				$functions[] = $modules[$opt]['callback'];
			}
		}

		// Bail if there are no functions to return.
		if( ! $functions ) return;

		// Return the output.
		if( $echo ) {
			foreach( $functions as $func ) call_user_func( $func );
		} else {
			ob_start();
			foreach( $functions as $func ) call_user_func( $func );
			return ob_get_clean();
		}
	}

	/**
     * Wrapper method for any html
	 *
	 * @since 	unknown
	 * @version	3.7.8
	 *
	 * @param	string	context		required ( used by generate_attr's dynamic filter )
	 * @param 	mixed	function	required ( the function called to be wrapped )
	 * @param 	array	args		required ( wrappers array - must have tag/attrs key defined )
	 * @param 	array	func_args	optional ( optional params for the function called above )
	 *
	 * @return 	string/object 	HTML/WP_Error
	 */
	public static function wrap( string $context, array $wrappers, $content = '', $func_args = [], $echo = true ) {
		// Set defaults.
		$defaults = [
			[
				'tag' 	=> 'div',
				'attrs' => [
					'class' => esc_attr( $context )
				]
			]
		];

		/**
		 * Merge and filter
		 * @since 3.7.8
		 */
		$args = apply_filters( "wecodeart/filter/wrappers/{$context}", $wrappers );

		// Wrapper must have a 'tag' defined
		$_wrappers = array_map( function( $item ) {
			return array_key_exists( 'tag', $item ) && is_string( $item['tag'] ); 
		}, $args );

		// Make sure $args are an array.
		if ( empty( $args ) || ( ! empty( $args ) && count( array_unique( $_wrappers ) ) !== 1 ) ) {
			$args = $defaults;
			new \WP_Error( 'wecodeart_markup_wrap_fallback', 
				sprintf( 
					__( 'Wrappers are not properly defined for "%s". Please check your theme code.', 'wecodeart' ),
					$context
				)
			);
		}
		
		// Build the HTML.
		$function_html = '';

		if( is_callable( $content ) ) {
			ob_start();
			call_user_func_array( $content, $func_args );
			$function_html .= ob_get_clean();
		} else {
			$function_html .= $content;
		}

		if( empty( $function_html ) ) return;

		$html = '';
		
		foreach( $args as $key => $elem ) {
			$_context 	= $context . '/' . $key; // Dynamic context filter for each wrapper.
			$attrs 		= isset( $elem['attrs'] ) ? ( array ) $elem['attrs'] : [];
			$open_tag 	= trim( implode( ' ', [ esc_html( $elem['tag'] ), self::generate_attr( $_context, $attrs ) ] ) );
			$html .= '<' . $open_tag . '>';
		}

		$html .= $function_html;
		
		/**
		 * Close the wrappers in reverse order
		 * @since 3.7.2
		 */
		$_args = array_reverse( $args );

		foreach( $_args as $key => $elem ) {
			$html .= '</' . esc_html( $elem['tag'] ) . '>';
		}

		/**
		 * Developer Comments
		 * @since 3.7.0
		 */
		if( WP_DEBUG === true ) {
			if( isset( $args[0]['attrs'] ) && isset( $args[0]['attrs']['class'] ) ) {
				$classes = explode( ' ', $args[0]['attrs']['class'] );
				$comment = ( count( $classes ) > 0 ) ? $classes[0] : $args[0]['attrs']['class'];
				$comment .= " @filter = `wecodeart/filter/wrappers/{$context}`";
				$html .= "<!-- /.{$comment} -->";
			}
		}

		/**
		 * Filter the final HTML output of the function
		 * @since 3.6.0
		 */
		$output = apply_filters( "wecodeart/filter/wrap/{$context}/output", $html, $context );

		// Return the output.
		if( $echo ) echo $output; 
		else return $output;
	}

	/**
	 * Renders template file with data.
	 *
	 * @since	3.7.3
	 *
	 * @param  string $file Relative path to the template file.
	 * @param  array  $data Dataset for the template.
	 *
	 * @return void
	 */
	public static function template( $file, $data = [] ) {
		$template = new Markup\Template( [
			'directory' => get_stylesheet_directory(),
			'templates' => 'views',
			'extension' => '.tpl.php'
		] );

		return $template->set_file( $file )->render( $data );
	}
}